@@ -24,6 +24,8 @@ import socket
 from enum import IntEnum, unique
 
 import nbxmpp
+from nbxmpp.namespaces import Namespace
+from nbxmpp.util import generate_id
 
 from gajim.common import app
 
@@ -74,7 +76,6 @@ class JingleTransport:
         """
         Build a candidate stanza for the given candidate
         """
-        pass
 
     def make_transport(self, candidates=None):
         """
@@ -97,7 +98,7 @@ class JingleTransport:
     def set_connection(self, conn):
         self.connection = conn
         if not self.sid:
-            self.sid = self.connection.connection.getAnID()
+            self.sid = generate_id()
 
     def set_file_props(self, file_props):
         self.file_props = file_props
@@ -143,7 +144,7 @@ class JingleTransportSocks5(JingleTransport):
             transport = JingleTransport.make_transport(self, candidates)
         else:
             transport = nbxmpp.Node('transport')
-        transport.setNamespace(nbxmpp.NS_JINGLE_BYTESTREAM)
+        transport.setNamespace(Namespace.JINGLE_BYTESTREAM)
         transport.setAttr('sid', self.sid)
         if self.file_props.dstaddr:
             transport.setAttr('dstaddr', self.file_props.dstaddr)
@@ -184,12 +185,12 @@ class JingleTransportSocks5(JingleTransport):
                 self.candidates.append(cand)
 
     def _add_local_ips_as_candidates(self):
-        if not app.config.get_per('accounts', self.connection.name,
-        'ft_send_local_ips'):
+        if not app.settings.get_account_setting(self.connection.name,
+                                                'ft_send_local_ips'):
             return
         if not self.connection:
             return
-        port = int(app.config.get('file_transfers_port'))
+        port = int(app.settings.get('file_transfers_port'))
         #type preference of connection type. XEP-0260 section 2.2
         type_preference = 126
         priority = (2**16) * type_preference
@@ -197,16 +198,21 @@ class JingleTransportSocks5(JingleTransport):
         hosts = set()
         local_ip_cand = []
 
-        candidate = {
-            'host': self.connection.peerhost[0],
-            'candidate_id': self.connection.connection.getAnID(),
-            'port': port,
-            'type': 'direct',
-            'jid': self.ourjid,
-            'priority': priority
-        }
-        hosts.add(self.connection.peerhost[0])
-        local_ip_cand.append(candidate)
+        my_ip = self.connection.local_address
+        if my_ip is None:
+            log.warning('No local address available')
+
+        else:
+            candidate = {
+                'host': my_ip,
+                'candidate_id': generate_id(),
+                'port': port,
+                'type': 'direct',
+                'jid': self.ourjid,
+                'priority': priority
+            }
+            hosts.add(my_ip)
+            local_ip_cand.append(candidate)
 
         try:
             for addrinfo in socket.getaddrinfo(socket.gethostname(), None):
@@ -215,7 +221,7 @@ class JingleTransportSocks5(JingleTransport):
                 addr != '::1':
                     candidate = {
                         'host': addr,
-                        'candidate_id': self.connection.connection.getAnID(),
+                        'candidate_id': generate_id(),
                         'port': port,
                         'type': 'direct',
                         'jid': self.ourjid,
@@ -239,7 +245,7 @@ class JingleTransportSocks5(JingleTransport):
                             continue
                         candidate = {
                             'host': addr,
-                            'candidate_id': self.connection.connection.getAnID(),
+                            'candidate_id': generate_id(),
                             'port': port,
                             'type': 'direct',
                             'jid': self.ourjid,
@@ -257,7 +263,7 @@ class JingleTransportSocks5(JingleTransport):
                             continue
                         candidate = {
                             'host': addr,
-                            'candidate_id': self.connection.connection.getAnID(),
+                            'candidate_id': generate_id(),
                             'port': port,
                             'type': 'direct',
                             'jid': self.ourjid,
@@ -279,15 +285,15 @@ class JingleTransportSocks5(JingleTransport):
         type_preference = 126
         priority = (2**16) * type_preference
         additional_ip_cand = []
-        port = int(app.config.get('file_transfers_port'))
-        ft_add_hosts = app.config.get('ft_add_hosts_to_send')
+        port = int(app.settings.get('file_transfers_port'))
+        ft_add_hosts = app.settings.get('ft_add_hosts_to_send')
 
         if ft_add_hosts:
             hosts = [e.strip() for e in ft_add_hosts.split(',')]
             for host in hosts:
                 candidate = {
                     'host': host,
-                    'candidate_id': self.connection.connection.getAnID(),
+                    'candidate_id': generate_id(),
                     'port': port,
                     'type': 'direct',
                     'jid': self.ourjid,
@@ -306,7 +312,7 @@ class JingleTransportSocks5(JingleTransport):
         priority = (2**16) * type_preference
         proxy_cand = []
         socks5conn = self.connection
-        proxyhosts = socks5conn._get_file_transfer_proxies_from_config(self.file_props)
+        proxyhosts = socks5conn.get_module('Bytestream')._get_file_transfer_proxies_from_config(self.file_props)
 
         if proxyhosts:
             self.file_props.proxyhosts = proxyhosts
@@ -314,7 +320,7 @@ class JingleTransportSocks5(JingleTransport):
             for proxyhost in proxyhosts:
                 candidate = {
                     'host': proxyhost['host'],
-                    'candidate_id': self.connection.connection.getAnID(),
+                    'candidate_id': generate_id(),
                     'port': int(proxyhost['port']),
                     'type': 'proxy',
                     'jid': proxyhost['jid'],
@@ -327,8 +333,8 @@ class JingleTransportSocks5(JingleTransport):
         self._add_candidates(proxy_cand)
 
     def get_content(self):
-        sesn = self.connection.get_jingle_session(self.ourjid,
-                                                  self.file_props.sid)
+        sesn = self.connection.get_module('Jingle').get_jingle_session(
+            self.ourjid, self.file_props.sid)
         for content in sesn.contents.values():
             if content.transport == self:
                 return content
@@ -338,15 +344,15 @@ class JingleTransportSocks5(JingleTransport):
         # send activate request to proxy, send activated confirmation to peer
         if not self.connection:
             return
-        sesn = self.connection.get_jingle_session(self.ourjid,
-                                                  self.file_props.sid)
+        sesn = self.connection.get_module('Jingle').get_jingle_session(
+            self.ourjid, self.file_props.sid)
         if sesn is None:
             return
 
         iq = nbxmpp.Iq(to=proxy['jid'], frm=self.ourjid, typ='set')
         auth_id = "au_" + proxy['sid']
         iq.setID(auth_id)
-        query = iq.setTag('query', namespace=nbxmpp.NS_BYTESTREAM)
+        query = iq.setTag('query', namespace=Namespace.BYTESTREAM)
         query.setAttr('sid', proxy['sid'])
         activate = query.setTag('activate')
         activate.setData(sesn.peerjid)
@@ -359,7 +365,7 @@ class JingleTransportSocks5(JingleTransport):
         content_object = self.get_content()
         content.setAttr('name', content_object.name)
         transport = nbxmpp.Node('transport')
-        transport.setNamespace(nbxmpp.NS_JINGLE_BYTESTREAM)
+        transport.setNamespace(Namespace.JINGLE_BYTESTREAM)
         transport.setAttr('sid', proxy['sid'])
         activated = nbxmpp.Node('activated')
         cid = None
@@ -400,7 +406,7 @@ class JingleTransportIBB(JingleTransport):
     def make_transport(self):
 
         transport = nbxmpp.Node('transport')
-        transport.setNamespace(nbxmpp.NS_JINGLE_IBB)
+        transport.setNamespace(Namespace.JINGLE_IBB)
         transport.setAttr('block-size', self.block_sz)
         transport.setAttr('sid', self.sid)
         return transport
@@ -443,7 +449,7 @@ class JingleTransportICEUDP(JingleTransport):
 
     def make_transport(self, candidates=None):
         transport = JingleTransport.make_transport(self, candidates)
-        transport.setNamespace(nbxmpp.NS_JINGLE_ICE_UDP)
+        transport.setNamespace(Namespace.JINGLE_ICE_UDP)
         if self.candidates and self.candidates[0].username and \
                 self.candidates[0].password:
             transport.setAttr('ufrag', self.candidates[0].username)
@@ -491,6 +497,6 @@ class JingleTransportICEUDP(JingleTransport):
         self.remote_candidates.extend(candidates)
         return candidates
 
-transports[nbxmpp.NS_JINGLE_ICE_UDP] = JingleTransportICEUDP
-transports[nbxmpp.NS_JINGLE_BYTESTREAM] = JingleTransportSocks5
-transports[nbxmpp.NS_JINGLE_IBB] = JingleTransportIBB
+transports[Namespace.JINGLE_ICE_UDP] = JingleTransportICEUDP
+transports[Namespace.JINGLE_BYTESTREAM] = JingleTransportSocks5
+transports[Namespace.JINGLE_IBB] = JingleTransportIBB
