@@ -15,28 +15,28 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import os
 import math
 import logging
 
-import cssutils
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import Pango
+import css_parser
 
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common.const import StyleAttr, CSSPriority
 
-from gajim.gtk.const import Theme
+from .const import Theme
 
-log = logging.getLogger('gajim.gtk.css')
+log = logging.getLogger('gajim.gui.css')
 settings = Gtk.Settings.get_default()
 
 
 class CSSConfig():
     def __init__(self):
-        """CSSConfig handles loading and storing of all relevant Gajim style files
+        """
+        CSSConfig handles loading and storing of all relevant Gajim style files
 
         The order in which CSSConfig loads the styles
 
@@ -51,7 +51,7 @@ class CSSConfig():
 
         # gajim-dark.css
 
-        Has only entrys which we want to override in gajim.css
+        Has only entries which we want to override in gajim.css
 
         # default.css or default-dark.css
 
@@ -66,7 +66,7 @@ class CSSConfig():
         css here."""
 
         # Delete empty rules
-        cssutils.ser.prefs.keepEmptyRules = False
+        css_parser.ser.prefs.keepEmptyRules = False
 
         # Holds the currently selected theme in the Theme Editor
         self._pre_css = None
@@ -84,6 +84,16 @@ class CSSConfig():
         # User Theme CSS Provider
         self._provider = Gtk.CssProvider()
 
+        # Used for dynamic classes like account colors
+        self._dynamic_provider = Gtk.CssProvider()
+        self._dynamic_dict = {}
+        self.refresh()
+
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            self._dynamic_provider,
+            CSSPriority.APPLICATION)
+
         # Cache of recently requested values
         self._cache = {}
 
@@ -96,6 +106,7 @@ class CSSConfig():
         self._load_default()
         self._load_selected()
         self._activate_theme()
+
         Gtk.StyleContext.add_provider_for_screen(
             Gdk.Screen.get_default(),
             self._provider,
@@ -103,19 +114,18 @@ class CSSConfig():
 
     @property
     def prefer_dark(self):
-        setting = app.config.get('dark_theme')
+        setting = app.settings.get('dark_theme')
         if setting == Theme.SYSTEM:
             if settings is None:
                 return False
             return settings.get_property('gtk-application-prefer-dark-theme')
         return setting == Theme.DARK
 
-    @staticmethod
-    def set_dark_theme(value=None):
+    def set_dark_theme(self, value=None):
         if value is None:
-            value = app.config.get('dark_theme')
+            value = app.settings.get('dark_theme')
         else:
-            app.config.set('dark_theme', value)
+            app.settings.set('dark_theme', value)
 
         if settings is None:
             return
@@ -123,6 +133,7 @@ class CSSConfig():
             settings.reset_property('gtk-application-prefer-dark-theme')
             return
         settings.set_property('gtk-application-prefer-dark-theme', bool(value))
+        self._load_css()
 
     def _load_css(self):
         self._load_css_from_file('gajim.css', CSSPriority.APPLICATION)
@@ -136,10 +147,10 @@ class CSSConfig():
                                      CSSPriority.DEFAULT_THEME_DARK)
 
     def _load_css_from_file(self, filename, priority):
-        path = os.path.join(configpaths.get('STYLE'), filename)
+        path = configpaths.get('STYLE') / filename
         try:
-            with open(path, "r") as f:
-                css = f.read()
+            with open(path, "r") as file_:
+                css = file_.read()
         except Exception as exc:
             log.error('Error loading css: %s', exc)
             return
@@ -152,6 +163,9 @@ class CSSConfig():
             Gtk.StyleContext.add_provider_for_screen(Gdk.Screen.get_default(),
                                                      provider,
                                                      priority)
+            self._load_selected()
+            self._activate_theme()
+
         except Exception:
             log.exception('Error loading application css')
 
@@ -167,8 +181,8 @@ class CSSConfig():
         return int(math.ceil(number / 100.0)) * 100
 
     def _gather_available_themes(self):
-        files = os.listdir(configpaths.get('MY_THEME'))
-        self.themes = [file[:-4] for file in files if file.endswith('.css')]
+        files = configpaths.get('MY_THEME').iterdir()
+        self.themes = [file.stem for file in files if file.suffix == '.css']
         if 'default' in self.themes:
             # Ignore user created themes that are named 'default'
             self.themes.remove('default')
@@ -178,20 +192,20 @@ class CSSConfig():
             theme = 'default-dark'
 
         if user:
-            return os.path.join(configpaths.get('MY_THEME'), '%s.css' % theme)
-        return os.path.join(configpaths.get('STYLE'), '%s.css' % theme)
+            return configpaths.get('MY_THEME') / f'{theme}.css'
+        return configpaths.get('STYLE') / f'{theme}.css'
 
     def _determine_theme_path(self):
         # Gets the path of the currently active theme.
         # If it does not exist, it falls back to the default theme
-        theme = app.config.get('roster_theme')
+        theme = app.settings.get('roster_theme')
         if theme == 'default':
             return self.get_theme_path(theme, user=False)
 
         theme_path = self.get_theme_path(theme)
-        if not theme or not os.path.exists(theme_path):
+        if not theme or not theme_path.exists():
             log.warning('Theme %s not found, fallback to default', theme)
-            app.config.set('roster_theme', 'default')
+            app.settings.set('roster_theme', 'default')
             log.info('Use Theme: default')
             return self.get_theme_path('default', user=False)
         log.info('Use Theme: %s', theme)
@@ -202,16 +216,16 @@ class CSSConfig():
             self._css_path = self._determine_theme_path()
         else:
             self._css_path = new_path
-        self._css = cssutils.parseFile(self._css_path)
+        self._css = css_parser.parseFile(self._css_path)
 
     def _load_default(self):
         self._default_css_path = self.get_theme_path('default', user=False)
-        self._default_css = cssutils.parseFile(self._default_css_path)
+        self._default_css = css_parser.parseFile(self._default_css_path)
 
     def _load_pre(self, theme):
         log.info('Preload theme %s', theme)
         self._pre_css_path = self.get_theme_path(theme)
-        self._pre_css = cssutils.parseFile(self._pre_css_path)
+        self._pre_css = css_parser.parseFile(self._pre_css_path)
 
     def _write(self, pre):
         path = self._css_path
@@ -247,14 +261,15 @@ class CSSConfig():
                 if not pre:
                     self._add_to_cache(selector, attr, value)
                 self._write(pre)
-                return
+                return None
 
         # The rule was not found, so we add it to this theme
         log.info('Set %s %s %s', selector, attr, value)
-        rule = cssutils.css.CSSStyleRule(selectorText=selector)
+        rule = css_parser.css.CSSStyleRule(selectorText=selector)
         rule.style[attr] = value
         css.add(rule)
         self._write(pre)
+        return None
 
     def set_font(self, selector, description, pre=False):
         css = self._css
@@ -282,7 +297,7 @@ class CSSConfig():
         # The rule was not found, so we add it to this theme
         log.info('Set Font for: %s %s %s %s %s',
                  selector, family, size, style, weight)
-        rule = cssutils.css.CSSStyleRule(selectorText=selector)
+        rule = css_parser.css.CSSStyleRule(selectorText=selector)
         rule.style['font-family'] = family
         rule.style['font-style'] = style
         rule.style['font-size'] = '%spt' % size
@@ -297,13 +312,14 @@ class CSSConfig():
         family = description.get_family()
         return family, size, style, weight
 
-    def _get_default_rule(self, selector, attr):
+    def _get_default_rule(self, selector, _attr):
         for rule in self._default_css:
             if rule.type != rule.STYLE_RULE:
                 continue
             if rule.selectorText == selector:
                 log.info('Get Default Rule %s', selector)
                 return rule
+        return None
 
     def get_font(self, selector, pre=False):
         if pre:
@@ -316,7 +332,7 @@ class CSSConfig():
                 pass
 
         if css is None:
-            return
+            return None
 
         for rule in css:
             if rule.type != rule.STYLE_RULE:
@@ -334,9 +350,12 @@ class CSSConfig():
                     self._add_to_cache(selector, 'fontdescription', desc)
                 return desc
 
+        self._add_to_cache(selector, 'fontdescription', None)
+        return None
+
     def _get_description_from_css(self, family, size, style, weight):
         if family is None:
-            return
+            return None
         desc = Pango.FontDescription()
         desc.set_family(family)
         if weight is not None:
@@ -394,13 +413,14 @@ class CSSConfig():
                         self._add_to_cache(selector, attr, value)
                     return value
 
-        # We didnt find the selector in the selected theme
+        # We didn’t find the selector in the selected theme
         # search in default theme
         if not pre:
             rule = self._get_default_rule(selector, attr)
-            if rule is not None:
-                self._add_to_cache(selector, attr, rule.style[attr])
-                return rule.style[attr]
+            value = rule if rule is None else rule.style[attr]
+            self._add_to_cache(selector, attr, value)
+            return value
+        return None
 
     def remove_value(self, selector, attr, pre=False):
         if attr == StyleAttr.FONT:
@@ -421,6 +441,7 @@ class CSSConfig():
                 rule.style.removeProperty(attr)
                 break
         self._write(pre)
+        return None
 
     def remove_font(self, selector, pre=False):
         css = self._css
@@ -442,18 +463,18 @@ class CSSConfig():
     def change_theme(self, theme):
         user = not theme == 'default'
         theme_path = self.get_theme_path(theme, user=user)
-        if not os.path.exists(theme_path):
+        if not theme_path.exists():
             log.error('Change Theme: Theme %s does not exist', theme_path)
             return False
         self._load_selected(theme_path)
         self._activate_theme()
-        app.config.set('roster_theme', theme)
+        app.settings.set('roster_theme', theme)
         log.info('Change Theme: Successful switched to %s', theme)
         return True
 
     def change_preload_theme(self, theme):
         theme_path = self.get_theme_path(theme)
-        if not os.path.exists(theme_path):
+        if not theme_path.exists():
             log.error('Change Preload Theme: Theme %s does not exist',
                       theme_path)
             return False
@@ -472,7 +493,7 @@ class CSSConfig():
 
         old_theme_path = self.get_theme_path(old_theme)
         new_theme_path = self.get_theme_path(new_theme)
-        os.rename(old_theme_path, new_theme_path)
+        old_theme_path.rename(new_theme_path)
         self.themes.remove(old_theme)
         self.themes.append(new_theme)
         self._load_pre(new_theme)
@@ -487,7 +508,7 @@ class CSSConfig():
 
     def add_new_theme(self, theme):
         theme_path = self.get_theme_path(theme)
-        if os.path.exists(theme_path):
+        if theme_path.exists():
             log.error('Add Theme: %s exists already', theme_path)
             return False
         with open(theme_path, 'w', encoding='utf8'):
@@ -498,8 +519,8 @@ class CSSConfig():
 
     def remove_theme(self, theme):
         theme_path = self.get_theme_path(theme)
-        if os.path.exists(theme_path):
-            os.remove(theme_path)
+        if theme_path.exists():
+            theme_path.unlink()
             self.themes.remove(theme)
         log.info('Remove Theme: Successful removed theme %s', theme)
 
@@ -511,3 +532,17 @@ class CSSConfig():
 
     def _invalidate_cache(self):
         self._cache = {}
+
+    def refresh(self):
+        css = ''
+        accounts = app.settings.get_accounts()
+        for index, account in enumerate(accounts):
+            color = app.settings.get_account_setting(account, 'account_color')
+            css_class = 'gajim_class_%s' % index
+            css += '.%s { background-color: %s }\n' % (css_class, color)
+            self._dynamic_dict[account] = css_class
+
+        self._dynamic_provider.load_from_data(css.encode())
+
+    def get_dynamic_class(self, name):
+        return self._dynamic_dict[name]
