@@ -108,6 +108,7 @@ class MessageInputTextView(Gtk.TextView):
             Gdk.DragAction.DEFAULT)
 
     def _on_focus_in(self, _widget, _event):
+        self.toggle_speller(True)
         scrolled = self.get_parent()
         scrolled.get_style_context().add_class('message-input-focus')
         return False
@@ -115,6 +116,8 @@ class MessageInputTextView(Gtk.TextView):
     def _on_focus_out(self, _widget, _event):
         scrolled = self.get_parent()
         scrolled.get_style_context().remove_class('message-input-focus')
+        if not self.has_text():
+            self.toggle_speller(False)
         return False
 
     def insert_text(self, text):
