@@ -12,7 +12,7 @@ cd "${DIR}"
 
 # CONFIG START
 MAJOR_PY_VERSION="3"
-MINOR_PY_VERSION="8"
+MINOR_PY_VERSION="9"
 PYTHON_VERSION="${MAJOR_PY_VERSION}.${MINOR_PY_VERSION}"
 BUILD_VERSION="0"
 
@@ -92,7 +92,8 @@ function install_deps {
         mingw-w64-"${ARCH}"-python-pyopenssl \
         mingw-w64-"${ARCH}"-python-certifi \
         mingw-w64-"${ARCH}"-python-six \
-        mingw-w64-"${ARCH}"-python-pygments
+        mingw-w64-"${ARCH}"-python-pygments \
+        mingw-w64-"${ARCH}"-python-gssapi
 
         build_pip install precis-i18n
 
